# Copyright (c) 2002 Juri Pakaste
# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

cvs_id = "$Id: SummaryParser.py,v 1.28 2003/07/21 15:47:29 juri Exp $"

import types
from xml.sax import saxutils, make_parser, SAXParseException
from xml.sax.handler import feature_namespaces, feature_namespace_prefixes
from StringIO import StringIO
import xml.sax._exceptions
import rfc822
import time
import urlparse
import traceback
from mx import DateTime
from mx.DateTime.Parser import DateTimeFromString, DateFromString
import SummaryParserCommon
import straw
from error import log

# various xml namespaces we use
BLOGCHANNEL_NS = "http://backend.userland.com/blogChannelModule"
CONTENT_NS = "http://purl.org/rss/1.0/modules/content/"
DUBLINCORE_NS = "http://purl.org/dc/elements/1.1/"
FRESHMEAT_NS = "http://freshmeat.net/rss/fm/"
RSS1_NS = "http://purl.org/rss/1.0/"
RSS2_NS = "http://backend.userland.com/rss2"
RSS09_NS = "http://my.netscape.com/rdf/simple/0.9/"
RDFCC_NS = "http://web.resource.org/cc/"
RSSCC_NS = "http://backend.userland.com/creativeCommonsRssModule"
RDF_NS = "http://www.w3.org/1999/02/22-rdf-syntax-ns#"
NECHO_NS_LIST = ("http://example.com/newformat#", "uri/of/echo/namespace#")
ENT_NS = "http://www.purl.org/NET/ENT/1.0/"
PRISM_NS = "http://prismstandard.org/namespaces/1.2/basic/"

class SummaryFormatException(Exception):
    pass

def isRSSNamespace(uri):
    return uri in (None, RSS09_NS, RSS1_NS, RSS2_NS)

def readISODate(text):
    text = text.strip()
    d = None
    try:
        d = DateTimeFromString(text)
    except:
        try:
            d = DateFromString(text)
        except:
            pass
    return d

class HandlerBase(saxutils.DefaultHandler):
    def __init__(self, feed):
        self._summary = straw.ParsedSummary()
        self._feed = feed
        self._content = ""
        self._attrs = []
        self._in_item = 0
        self._in_textinput = 0
        self._in_image = 0
        self._in_description = 0
        self._item = None
        self._element = None
        self._default_license_urls = []

    def characters(self, ch):
        self._content += ch
        return

    def get_summary(self):
        return self._summary

class RSSHandler(HandlerBase):
    def startElementNS(self, (uri, name), qname, attrs):
        if uri in NECHO_NS_LIST:
            raise SummaryFormatException, "Necho format detected"
        elif name == 'feed':
            raise SummaryFormatException, "Necho format detected"

        if not self._in_item and name == 'item':
            self._in_item = 1
            self._item = straw.SummaryItem()
        elif name == 'textinput':
            self._in_textinput = 1
        elif name == 'image':
            self._in_image = 1
        elif self._in_item and isRSSNamespace(uri) and name == 'description':
            self._in_description = 1
        elif self._in_description:
            raise ValueError, 'No elements allowed in description'
        self._content = ""
        self._attrs.append(attrs)

    def endElementNS(self, (uri, name), qname):
        attrs = self._attrs.pop()
        if self._in_item:
            if isRSSNamespace(uri):
                if name == 'title':
                    self._item.title = self._content.strip()
                elif name == 'description':
                    self._in_description = 0
                    if self._item.description is None:
                        self._item.description = self._content.strip()
                elif name == 'link':
                    self._item.link = self._content.strip()
                elif name == 'guid':
                    self._item.guid = self._content.strip()
                elif name == 'pubDate':
                    try:
                        self._item.pub_date = DateTimeFromString(
                            self._content.strip(), ('lit',))
                    except Exception, e:
                        self._item.pub_date = None
                elif name == 'source':
                    try:
                        self._item.source = {'url': attrs[(None, u'url')],
                                             'text': self._content.strip()}
                    except KeyError, e:
                        pass
                elif name == 'date':
                    self._item.pub_date = readISODate(self._content)
                elif name == 'item':
                    self._in_item = 0
                    if self._item.description is not None:
                        idh = SummaryParserCommon.ItemDescriptionHandler(self._feed)
                        try:
                            idh.feed(self._item.description)
                            images = idh.get_images()
                            for i in images:
                                self._item.set_image(i, None)
                            description = idh.get_data()
                            self._item.description = description
                        except Exception, ex:
                            pass
                    if self._item.license_urls == []:
                        self._item.license_urls = self._default_license_urls[:]
                    if attrs.has_key((RDF_NS, 'about')) and not self._item.link:
                        self._item.link = attrs[(RDF_NS, 'about')]
                    self._summary.addItem(self._item)
                    self._item = None
            elif uri == FRESHMEAT_NS:
                if name == 'license':
                    self._item.fm_license = self._content.strip()
                elif name == 'changes':
                    self._item.fm_changes = self._content.strip()
            elif uri == CONTENT_NS:
                if name == 'encoded':
                    self._item.description = self._content.strip()
            elif uri == DUBLINCORE_NS:
                if name == 'date':
                    self._item.pub_date = readISODate(self._content)
                elif name == 'creator':
                    self._item.creator = self._content.strip()
            elif uri == RDFCC_NS:
                if name == "license":
                    u = attrs[(RDF_NS, 'resource')]
                    if u is not None:
                        self._item.license_urls.append(u)
            elif uri == RSSCC_NS:
                if name == "license":
                    self._item.license_urls.append(self._content.strip())
            elif uri == PRISM_NS:
                if name == "publicationName":
                    self._item.publication_name = self._content.strip()
                elif name == "volume":
                    self._item.publication_volume = self._content.strip()
                elif name == "number":
                    self._item.publication_number = self._content.strip()
                elif name == "section":
                    self._item.publication_section = self._content.strip()
                elif name == "startingPage":
                    self._item.publication_starting_page = self._content.strip()
        elif self._in_textinput:
            if name == 'textinput':
                self._in_textinput = 0
        elif self._in_image:
            if name == 'image':
                self._in_image = 0
        else:
            # channel head
            if uri in (None, RSS09_NS, RSS1_NS, RSS2_NS):
                if name == 'title':
                    self._summary.title = self._content.strip()
                elif name == 'link':
                    self._summary.link = self._content.strip()
                elif name == 'description':
                    self._summary.description = self._content.strip()
                elif name == 'language':
                    self._summary.language = self._content.strip()
                elif name == 'lastBuildDate':
                    try:
                        self._summary.last_build_date = DateTimeFromString(
                            self._content.strip(), ('iso',))
                    except:
                        self._summary.last_build_date = DateTime.now()
                elif name == 'copyright':
                    self._summary.copyright = self._content.strip()
            elif uri == DUBLINCORE_NS:
                if name == 'date':
                    try:
                        self._summary.last_build_date = DateTimeFromString(
                            self._content.strip(), ('iso',))
                    except:
                        self._summary.last_build_date = DateTime.now()
                elif name == 'rights':
                    self._summary.copyright = self._content.strip()
                elif name == 'creator':
                    self._summary.creator = self._content.strip()
            elif uri == BLOGCHANNEL_NS:
                if name == 'blogRoll':
                    self._summary.blogroll = self._content.strip()
                elif name == 'mySubscriptions':
                    self._summary.subscriptions = self._content.strip()
                elif name == 'blink':
                    self._summary.blink = self._content.strip()
            elif uri == RDFCC_NS:
                if name == "license":
                    u = attrs[(RDF_NS, 'resource')]
                    if u is not None:
                        self._default_license_urls.append(u)
            elif uri == RSSCC_NS:
                if name == "license":
                    self._default_license_urls.append(self._content.strip())

class NechoHandler(HandlerBase):
    def startElementNS(self, (uri, name), qname, attrs):
        if name == 'rss':
            raise SummaryFormatException, 'RSS format detected'
        elif name == 'RDF':
            raise SummaryFormatException, 'RDF format detected'

        if not self._in_item and name == 'entry':
            self._in_item = 1
            self._item = straw.SummaryItem()
        elif self._in_item and name == 'content':
            self._in_description = 1
        self._content = ""
        self._attrs = attrs

    def endElementNS(self, (uri, name), qname):
        if self._in_item:
            if uri is None or uri in NECHO_NS_LIST:
                if name == 'entry':
                    self._in_item = 0

                    if self._item.description is not None:
                        idh = SummaryParserCommon.ItemDescriptionHandler(self._feed)
                        try:
                            idh.feed(self._item.description)
                            images = idh.get_images()
                            for i in images:
                                self._item.set_image(i, None)
                            description = idh.get_data()
                            self._item.description = description
                        except Exception, ex:
                            pass

                    self._summary.addItem(self._item)
                    self._item = None
                elif name == 'content':
                    self._in_description = 0
                    if self._item.description is None:
                        self._item.description = self._content.strip()
                elif name == 'title':
                    self._item.title = self._content.strip()
                elif name == 'link':
                    self._item.link = self._content.strip()
                elif name == 'id':
                    self._item.guid = self._content.strip()
                elif name in ('created', 'issued', 'modified'):
                    try:
                        parsed_date = DateTimeFromString(
                            self._content.strip(), ('iso',))
                    except:
                        parsed_date = DateTime.Epoch
                    if not isinstance(self._summary.last_build_date,
                                      types.IntType):
                        date_compare = DateTime.cmp(
                            self._summary.last_build_date, parsed_date)
                    else:
                        date_compare = -1
                    if date_compare < 0:
                        self._summary.last_build_date = parsed_date

            elif uri == ENT_NS:
                pass
        else:
            if name == 'title':
                self._summary.title = self._content.strip() + \
                                      self._summary.title
            elif name == 'subtitle':
                self._summary.title = self._summary.title + ': ' + \
                                      self._content.strip()
            elif name == 'link':
                self._summary.link = self._content.strip()
            elif name == 'modified':
                try:
                    self._summary.last_build_date = DateTimeFromString(
                        self._content.strip(), ('iso',))
                except:
                    self._summary.last_build_date = DateTime.now()
 
class SummaryParser:
    def __init__(self):
        pass

    def parse(self, content, feed):
        parser = make_parser()
        parser.setFeature(feature_namespaces, 1)

        enc = "utf-8"
        try:
            tenc = SummaryParserCommon.read_encoding(content)
            if tenc:
                enc = tenc
        except ValueError:
            pass

        handler = None
        for handler_class in (RSSHandler, NechoHandler):
            handler_test = handler_class(feed)
            parser.setContentHandler(handler_test)
            try:
                parser.parse(StringIO(content))
            except SummaryFormatException, ex:
                continue
            except SAXParseException, ex:
                raise straw.RSSParseException, "Invalid XML"
            except ValueError, ex:
                raise straw.RSSParseException, "Invalid RSS"

            handler = handler_test
            break

        if not handler:
            raise straw.RSSParseException, "Unknown summary format"

        parsed = handler.get_summary()
        for item in parsed.items:
            item.feed = feed
        return parsed
